/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/type/matcher/v3/status_code_input.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg_internal.h"
#include "envoy/type/matcher/v3/status_code_input.upb.h"
#include "udpa/annotations/status.upb.h"

#include "upb/port_def.inc"

const upb_MiniTable envoy_type_matcher_v3_HttpResponseStatusCodeMatchInput_msginit = {
  NULL,
  NULL,
  UPB_SIZE(0, 0), 0, kUpb_ExtMode_NonExtendable, 0, 255, 0,
};

const upb_MiniTable envoy_type_matcher_v3_HttpResponseStatusCodeClassMatchInput_msginit = {
  NULL,
  NULL,
  UPB_SIZE(0, 0), 0, kUpb_ExtMode_NonExtendable, 0, 255, 0,
};

static const upb_MiniTable *messages_layout[2] = {
  &envoy_type_matcher_v3_HttpResponseStatusCodeMatchInput_msginit,
  &envoy_type_matcher_v3_HttpResponseStatusCodeClassMatchInput_msginit,
};

const upb_MiniTable_File envoy_type_matcher_v3_status_code_input_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  2,
  0,
  0,
};

#include "upb/port_undef.inc"

