/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     xds/type/matcher/v3/cel.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg_internal.h"
#include "xds/type/matcher/v3/cel.upb.h"
#include "xds/annotations/v3/status.upb.h"
#include "xds/type/v3/cel.upb.h"
#include "validate/validate.upb.h"

#include "upb/port_def.inc"

static const upb_MiniTable_Sub xds_type_matcher_v3_CelMatcher_submsgs[1] = {
  {.submsg = &xds_type_v3_CelExpression_msginit},
};

static const upb_MiniTable_Field xds_type_matcher_v3_CelMatcher__fields[1] = {
  {1, UPB_SIZE(4, 8), UPB_SIZE(1, 1), 0, 11, kUpb_FieldMode_Scalar | (kUpb_FieldRep_Pointer << kUpb_FieldRep_Shift)},
};

const upb_MiniTable xds_type_matcher_v3_CelMatcher_msginit = {
  &xds_type_matcher_v3_CelMatcher_submsgs[0],
  &xds_type_matcher_v3_CelMatcher__fields[0],
  UPB_SIZE(8, 16), 1, kUpb_ExtMode_NonExtendable, 1, 255, 0,
};

static const upb_MiniTable *messages_layout[1] = {
  &xds_type_matcher_v3_CelMatcher_msginit,
};

const upb_MiniTable_File xds_type_matcher_v3_cel_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  1,
  0,
  0,
};

#include "upb/port_undef.inc"

